#include <iostream>
#include <fstream>
#include <boost/format.hpp>
#include <boost/optional.hpp>
#include <fileref.h>
#include <tstring.h>
#include <tag.h>
#include "abstract_tag.hpp"
#include "store.hpp"
using namespace std;
using namespace boost;

namespace {

    TagLib::String make_taglib_str(const optional<string>& utf8_str) {
        if (utf8_str) {
            return TagLib::String(*utf8_str, TagLib::String::UTF8);
        }
        else {
            return TagLib::String::null;
        }
    }

}

namespace TagMachine {

    void store_tag(
        const std::string& music_fpath,
        const std::string& tag_fpath,
        const std::string& tag_format) {

        ifstream tag_stream(
            tag_fpath == "-" ? "/dev/fd/0" : tag_fpath.c_str(),
            ios::in);

        if (!tag_stream.good()) {
            throw (format("failed to read tag file: %s") % tag_fpath).str();
        }

        TagLib::FileRef music(music_fpath.c_str());
        if (music.isNull()) {
            throw (format("failed to manipulate music file: %s") % music_fpath).str();
        }

        AbstractTag tag(tag_stream, force_find_format_by_name(tag_format));

        music.tag()->setTitle  (make_taglib_str(tag.title  ));
        music.tag()->setArtist (make_taglib_str(tag.artist ));
        music.tag()->setAlbum  (make_taglib_str(tag.album  ));
        music.tag()->setComment(make_taglib_str(tag.comment));
        music.tag()->setGenre  (make_taglib_str(tag.genre  ));
        music.tag()->setYear   (tag.year        ? *(tag.year       ) : 0);
        music.tag()->setTrack  (tag.tracknumber ? *(tag.tracknumber) : 0);

        music.save();
    }
    
}
